<?php
/**
 * @package Stag_Customizer
 *
 * @since  1.0.1.
 */

if ( ! function_exists( 'stag_get_font_stack' ) ) :
/**
 * Validate the font choice and get a font stack for it.
 *
 * @since  1.0.0.
 *
 * @param  string    $font    The 1st font in the stack.
 * @return string             The full font stack.
 */
function stag_get_font_stack( $font ) {
	$all_fonts = stag_get_all_fonts();

	// Sanitize font choice
	$font = stag_sanitize_font_choice( $font );

	// Standard font
	if ( isset( $all_fonts[ $font ]['stack'] ) && ! empty( $all_fonts[ $font ]['stack'] ) ) {
		$stack = $all_fonts[ $font ]['stack'];
	} elseif ( in_array( $font, stag_all_font_choices() ) ) {
		$stack = '"' . $font . '","Helvetica Neue",Helvetica,Arial,sans-serif';
	} else {
		$stack = '"Helvetica Neue",Helvetica,Arial,sans-serif';
	}

	/**
	 * Allow developers to filter the full font stack.
	 *
	 * @param string    $stack    The font stack.
	 * @param string    $font     The font.
	 */
	return apply_filters( 'stag_font_stack', $stack, $font );

	return $stack;
}
endif;

if ( ! function_exists( 'stag_get_relative_font_size' ) ) :
/**
 * Convert a font size to a relative size based on a starting value and percentage.
 *
 * @since  1.0.0.
 *
 * @param  mixed    $value         The value to base the final value on.
 * @param  mixed    $percentage    The percentage of change.
 * @return float                   The converted value.
 */
function stag_get_relative_font_size( $value, $percentage ) {
	return round( (float) $value * ( $percentage / 100 ) );
}
endif;

if ( ! function_exists( 'stag_convert_px_to_rem' ) ) :
/**
 * Given a px value, return a rem value.
 *
 * @since  1.0.0.
 *
 * @param  mixed    $px      The value to convert.
 * @param  mixed    $base    The font-size base for the rem conversion.
 * @return float             The converted value.
 */
function stag_convert_px_to_rem( $px, $base = 0 ) {
	return (float) $px / 10;
}
endif;

if ( ! function_exists( 'stag_get_font_property_option_keys' ) ) :
/**
 * Return all the option keys for the specified font property.
 *
 * @since  1.3.0.
 *
 * @param  string    $property    The font property to search for.
 * @return array                  Array of matching font option keys.
 */
function stag_get_font_property_option_keys( $property = 'font' ) {
	$mods = stag_get_theme_mods();
	$all_keys = array_keys( $mods['fonts'] );

	$font_keys = array();
	foreach ( $all_keys as $key ) {
		if ( preg_match( '/^' . $property . '-/', $key ) || preg_match( '/^font-' . $property . '-/', $key ) ) {
			$font_keys[] = $key;
		}
	}

	return $font_keys;
}
endif;

if ( ! function_exists( 'stag_get_google_font_uri' ) ) :
/**
 * Build the HTTP request URL for Google Fonts.
 *
 * @since  1.0.0.
 *
 * @return string    The URL for including Google Fonts.
 */
function stag_get_google_font_uri() {
	$keys = stag_get_font_property_option_keys();
	$fonts = array();

	foreach ( $keys as $key ) {
		$fonts[] = stag_theme_mod( 'fonts', $key );
	}

	// De-dupe the fonts
	$fonts         = array_unique( $fonts );
	$allowed_fonts = stag_get_google_fonts();
	$family        = array();

	// Validate each font and convert to URL format
	foreach ( $fonts as $font ) {
		$font = trim( $font );

		// Verify that the font exists
		if ( array_key_exists( $font, $allowed_fonts ) ) {
			// Build the family name and variant string (e.g., "Open+Sans:regular,italic,700")
			$family[] = urlencode( $font ) . ':' . join( ',', stag_choose_google_font_variants( $font, $allowed_fonts[ $font ]['variants'] ) );
		}
	}

	// Convert from array to string
	if ( empty( $family ) ) {
		return '';
	} else {
		$request = '//fonts.googleapis.com/css?family=' . implode( '|', $family );
	}

	// Load the font subset
	$subset = stag_theme_mod( 'fonts', 'subset' );

	if ( 'all' === $subset ) {
		$subsets_available = stag_get_google_font_subsets();

		// Remove the all set
		unset( $subsets_available['all'] );

		// Build the array
		$subsets = array_keys( $subsets_available );
	} else {
		$subsets = array(
			'latin',
			$subset,
		);
	}

	// Append the subset string
	if ( ! empty( $subsets ) ) {
		$request .= urlencode( '&subset=' . join( ',', $subsets ) );
	}

	/**
	 * Filter the Google Fonts URL.
	 *
	 * @param string    $url    The URL to retrieve the Google Fonts.
	 */
	return apply_filters( 'stag_get_google_font_uri', esc_url( $request ) );
}
endif;

if ( ! function_exists( 'stag_choose_google_font_variants' ) ) :
/**
 * Given a font, chose the variants to load for the theme.
 *
 * Attempts to load regular, italic, and 700. If regular is not found, the first variant in the family is chosen. italic
 * and 700 are only loaded if found. No fallbacks are loaded for those fonts.
 *
 * @since  1.0.0.
 *
 * @param  string    $font        The font to load variants for.
 * @param  array     $variants    The variants for the font.
 * @return array                  The chosen variants.
 */
function stag_choose_google_font_variants( $font, $variants = array() ) {
	$chosen_variants = array();
	if ( empty( $variants ) ) {
		$fonts = stag_get_google_fonts();

		if ( array_key_exists( $font, $fonts ) ) {
			$variants = $fonts[ $font ]['variants'];
		}
	}

	// If a "regular" variant is not found, get the first variant
	if ( ! in_array( 'regular', $variants ) ) {
		$chosen_variants[] = $variants[0];
	} else {
		$chosen_variants[] = 'regular';
	}

	// Only add "italic" if it exists
	if ( in_array( 'italic', $variants ) ) {
		$chosen_variants[] = 'italic';
	}

	// Only add "300" if it exists
	if ( in_array( '300', $variants ) ) {
		$chosen_variants[] = '300';
	}

	// Only add "700" if it exists
	if ( in_array( '700', $variants ) ) {
		$chosen_variants[] = '700';
	}

	// Only add "900" if it exists
	if ( in_array( '900', $variants ) ) {
		$chosen_variants[] = '900';
	}

	/**
	 * Allow developers to alter the font variant choice.
	 *
	 * @param array     $variants    The list of variants for a font.
	 * @param string    $font        The font to load variants for.
	 * @param array     $variants    The variants for the font.
	 */
	return apply_filters( 'stag_font_variants', array_unique( $chosen_variants ), $font, $variants );
}
endif;

if ( ! function_exists( 'stag_sanitize_font_subset' ) ) :
/**
 * Sanitize the Character Subset choice.
 *
 * @since  1.0.0
 *
 * @param  string    $value    The value to sanitize.
 * @return array               The sanitized value.
 */
function stag_sanitize_font_subset( $value ) {
	if ( ! array_key_exists( $value, stag_get_google_font_subsets() ) ) {
		$value = 'latin';
	}

	/**
	 * Filter the sanitized subset choice.
	 *
	 * @param string    $value    The chosen subset value.
	 */
	return apply_filters( 'stag_sanitize_font_subset', $value );
}
endif;

if ( ! function_exists( 'stag_get_google_font_subsets' ) ) :
/**
 * Retrieve the list of available Google font subsets.
 *
 * @since  1.0.0.
 *
 * @return array    The available subsets.
 */
function stag_get_google_font_subsets() {
	/**
	 * Filter the list of supported Google Font subsets.
	 *
	 * @param array    $subsets    The list of subsets.
	 */
	return apply_filters( 'stag_get_google_font_subsets', array(
		'all'          => esc_html__( 'All', 'carbon' ),
		'arabic'       => esc_html__( 'Arabic', 'carbon' ),
		'bengali'      => esc_html__( 'Bengali', 'carbon' ),
		'cyrillic'     => esc_html__( 'Cyrillic', 'carbon' ),
		'cyrillic-ext' => esc_html__( 'Cyrillic Extended', 'carbon' ),
		'devanagari'   => esc_html__( 'Devanagari', 'carbon' ),
		'greek'        => esc_html__( 'Greek', 'carbon' ),
		'greek-ext'    => esc_html__( 'Greek Extended', 'carbon' ),
		'gujarati'     => esc_html__( 'Gujarati', 'carbon' ),
		'gurumukhi'    => esc_html__( 'Gurumukhi', 'carbon' ),
		'hebrew'       => esc_html__( 'Hebrew', 'carbon' ),
		'japanese'     => esc_html__( 'Japanese', 'carbon' ),
		'kannada'      => esc_html__( 'Kannada', 'carbon' ),
		'khmer'        => esc_html__( 'Khmer', 'carbon' ),
		'korean'       => esc_html__( 'Korean', 'carbon' ),
		'latin'        => esc_html__( 'Latin', 'carbon' ),
		'latin-ext'    => esc_html__( 'Latin Extended', 'carbon' ),
		'malyalam'     => esc_html__( 'Malyalam', 'carbon' ),
		'myanmar'      => esc_html__( 'Myanmar', 'carbon' ),
		'oriya'        => esc_html__( 'Oriya', 'carbon' ),
		'sinhala'      => esc_html__( 'Sinhala', 'carbon' ),
		'tamil'        => esc_html__( 'Tamil', 'carbon' ),
		'telugu'       => esc_html__( 'Telugu', 'carbon' ),
		'thai'         => esc_html__( 'Thai', 'carbon' ),
		'vietnamese'   => esc_html__( 'Vietnamese', 'carbon' ),
	) );
}
endif;

if ( ! function_exists( 'stag_sanitize_font_choice' ) ) :
/**
 * Sanitize a font choice.
 *
 * @since  1.0.0.
 *
 * @param  string    $value    The font choice.
 * @return string              The sanitized font choice.
 */
function stag_sanitize_font_choice( $value ) {
	if ( ! is_string( $value ) ) {
		// The array key is not a string, so the chosen option is not a real choice
		return '';
	} else if ( array_key_exists( $value, stag_all_font_choices() ) ) {
		return $value;
	} else {
		return '';
	}

	/**
	 * Filter the sanitized font choice.
	 *
	 * @param string    $value    The chosen font value.
	 */
	return apply_filters( 'stag_sanitize_font_choice', $return );
}
endif;

if ( ! function_exists( 'stag_all_font_choices' ) ) :
/**
 * Packages the font choices into value/label pairs for use with the customizer.
 *
 * @since  1.0.0.
 *
 * @return array    The fonts in value/label pairs.
 */
function stag_all_font_choices() {
	$fonts   = stag_get_all_fonts();
	$choices = array();

	// Repackage the fonts into value/label pairs
	foreach ( $fonts as $key => $font ) {
		$choices[ $key ] = $font['label'];
	}

	return $choices;
}
endif;

if ( ! function_exists( 'stag_get_all_fonts' ) ) :
/**
 * Compile font options from different sources.
 *
 * @since  1.0.0.
 *
 * @return array    All available fonts.
 */
function stag_get_all_fonts() {
	$heading1            = array( 1 => array( 'label' => sprintf( '&mdash; %s &mdash;', esc_html__( 'Standard Fonts', 'carbon' ) ) ) );
	$standard_fonts      = stag_get_standard_fonts();

	$google_fonts        = stag_get_google_fonts();

	$serif_heading       = array( 2 => array( 'label' => sprintf( '&mdash; %s &mdash;', esc_html__( 'Serif Fonts (Google)', 'carbon' ) ) ) );
	$serif_fonts         = wp_list_filter( $google_fonts, array( 'category' => 'serif' ) );

	$sans_serif_heading  = array( 3 => array( 'label' => sprintf( '&mdash; %s &mdash;', esc_html__( 'Sans Serif Fonts (Google)', 'carbon' ) ) ) );
	$sans_serif_fonts    = wp_list_filter( $google_fonts, array( 'category' => 'sans-serif' ) );

	$display_heading     = array( 4 => array( 'label' => sprintf( '&mdash; %s &mdash;', esc_html__( 'Display Fonts (Google)', 'carbon' ) ) ) );
	$display_fonts       = wp_list_filter( $google_fonts, array( 'category' => 'display' ) );

	$handwriting_heading = array( 4 => array( 'label' => sprintf( '&mdash; %s &mdash;', esc_html__( 'Handwriting Fonts (Google)', 'carbon' ) ) ) );
	$handwriting_fonts   = wp_list_filter( $google_fonts, array( 'category' => 'handwriting' ) );

	$monospace_heading   = array( 4 => array( 'label' => sprintf( '&mdash; %s &mdash;', esc_html__( 'Monospace Fonts (Google)', 'carbon' ) ) ) );
	$monospace_fonts     = wp_list_filter( $google_fonts, array( 'category' => 'monospace' ) );

	return apply_filters( 'stag_all_fonts', array_merge(
		$heading1, $standard_fonts,
		$serif_heading, $serif_fonts,
		$sans_serif_heading, $sans_serif_fonts,
		$display_heading, $display_fonts,
		$handwriting_heading, $handwriting_fonts,
		$monospace_heading, $monospace_fonts
	) );
}
endif;

if ( ! function_exists( 'stag_get_standard_fonts' ) ) :
/**
 * Return an array of standard websafe fonts.
 *
 * @since  1.0.0.
 *
 * @return array    Standard websafe fonts.
 */
function stag_get_standard_fonts() {
	return array(
		'serif' => array(
			'label' => _x( 'Serif', 'font style', 'carbon' ),
			'stack' => 'Georgia,Times,"Times New Roman",serif',
		),
		'sans-serif' => array(
			'label' => _x( 'Sans Serif', 'font style', 'carbon' ),
			'stack' => '"Helvetica Neue",Helvetica,Arial,sans-serif',
		),
		'monospace' => array(
			'label' => _x( 'Monospaced', 'font style', 'carbon' ),
			'stack' => 'Monaco,"Lucida Sans Typewriter	","Lucida Typewriter","Courier New",Courier,monospace',
		)
	);
}
endif;
